/*
 * Client.
 * 
 * Tries to connect to the server. Prints the server's output.
 * 
 * We can create a connecting socket in three steps:
 * 
 * 1. Create a socket
 * 2. Connect to the server socket
 * 3. Start receiving
 * 
 */

#include <iostream>
#include <string>
#include <sys/types.h>
#ifdef WIN32
#include <winsock2.h>
#else
#include <sys/socket.h>
#include <arpa/inet.h>
#endif
#include <unistd.h>

using namespace std;

int main(int argc, char *argv[]) {
  int err;
  
  // Winsock2 is stupid...
  #ifdef WIN32
  WSADATA wsaData;
  err = WSAStartup(0x0002, &wsaData);
  if(err) {
    cerr << "Error starting Winsock2" << endl;
    return -1;
  }
  #endif
  
  // Step 1: Open a socket
  int sock;
  sock = socket(PF_INET, SOCK_STREAM, 0);

  if(!sock) {
    cerr << "Error while opening a socket!" <<endl;
    return -1;
  }
  
  // Step 2: Connect the socket to 127.0.0.1:4000
  struct sockaddr_in addr;
  memset(&addr, 0, sizeof(addr));
  addr.sin_family = AF_INET;
  addr.sin_addr.s_addr = inet_addr("127.0.0.1");
  addr.sin_port = htons(4000);
  
  err = connect(sock, (struct sockaddr*)&addr, sizeof(addr));
  if(err) {
    cerr << "Error while connecting the socket to an address!" << endl;
  }
  
  // Step 3: Start receiving
  char buffer[256];
  ssize_t size;
  while(0 < (size = recv(sock, buffer, sizeof(buffer) - 1, 0))) {
    buffer[size] = 0;
    printf("%s", buffer);

    // Send a confirmation to the server
    send(sock, "", 1, 0);
  }
    
  // Clean up. Close socket
  close(sock);
  
  #ifdef WIN32
  // Deinitialize Winsock2
  WSACleanup();
  #endif
  
  return 0;
}
