/*
 * Server.
 * 
 * Opens up a port. Says hello when connected.
 * 
 * We can create a listening socket in three steps:
 * 
 * 1. Create a socket
 * 2. Bind the socket to a network address
 *    (Firewalls will start to complain)
 * 3. Start listening
 * 
 */

#include <iostream>
#include <string>
#include <sys/types.h>
#ifdef WIN32
#include <winsock2.h>
#else
#include <sys/socket.h>
#include <arpa/inet.h>
#endif
#include <unistd.h>

using namespace std;

int main(int argc, char *argv[]) {
  int err;
  
  // Winsock2 is stupid...
  #ifdef WIN32
  WSADATA wsaData;
  err = WSAStartup(0x0002, &wsaData);
  if(err) {
    cerr << "Error starting Winsock2" << endl;
    return -1;
  }
  #endif
  
  // Step 1: Open a socket
  int sock;
  sock = socket(PF_INET, SOCK_STREAM, 0);

  if(!sock) {
    cerr << "Error while opening a socket!" <<endl;
    return -1;
  }
  
  // Step 2: Bind the socket to 0.0.0.0:4000
  struct sockaddr_in addr;
  memset(&addr, 0, sizeof(addr));
  addr.sin_family = AF_INET;
  addr.sin_addr.s_addr = 0;
  addr.sin_port = htons(4000);
  
  err = bind(sock, (struct sockaddr*)&addr, sizeof(addr));
  if(err) {
    cerr << "Error while binding the socket to an address!" << endl;
  }
  
  // Step 3: Start listening
  err = listen(sock, 10);
  if(err) {
    cerr << "Error while setting queue length!" << endl;
  }
  int accp = accept(sock, NULL, 0);
  
  // Say hello
  send(accp, "Hello, world!\n", 15, 0);
  
  // The client will not receive anything unless we wait for a confirmation
  char buffer[1];
  recv(accp, buffer, sizeof(buffer), 0);
  
  // Clean up. Close sockets
  close(accp);
  close(sock);
  
  #ifdef WIN32
  // Deinitialize Winsock2
  WSACleanup();
  #endif
  
  return 0;
}
